(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     24142,        511]
NotebookOptionsPosition[     20915,        456]
NotebookOutlinePosition[     21269,        472]
CellTagsIndexPosition[     21226,        469]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Example: code synthesis", "Subsection",
 CellChangeTimes->{{3.7191203670416203`*^9, 3.71912037087922*^9}, {
  3.729317197243264*^9, 3.7293172107850647`*^9}, {3.7294856657056656`*^9, 
  3.7294856682172976`*^9}},ExpressionUUID->"61718ed3-bdbb-4523-bbc2-\
370a0fc7fbc4"],

Cell["\<\
This notebook presents the capability of FMT to build models automatically, \
here by means of injecting code constructs into regular operation definitions.\
\>", "Text",
 CellChangeTimes->{{3.7293172201896644`*^9, 3.7293173407618647`*^9}, {
  3.7293175789972644`*^9, 3.7293175806664643`*^9}, {3.7294856740361724`*^9, 
  3.729485779714141*^9}, {3.7294871739117503`*^9, 3.7294871771253505`*^9}, {
  3.7294886801272507`*^9, 
  3.729488683655887*^9}},ExpressionUUID->"1db54fa5-be64-46d3-b176-\
848795c5d7f3"],

Cell["\<\
As a first example, the standard Mathematica function BooleanCountingFunction \
is used to generate complex Boolean expression that will become part of the \
model. This function creates expressions that evaluate to True when a given \
number of input variables are True - for example, a 4-variable expression \
that gives True when exactly two variables are True is:\
\>", "Text",
 CellChangeTimes->{{3.7294858000526037`*^9, 3.7294859018277607`*^9}, {
  3.7294859610364547`*^9, 3.7294860520557747`*^9}, {3.7294871570753503`*^9, 
  3.7294871642699504`*^9}, {3.729494181088128*^9, 
  3.729494182928928*^9}},ExpressionUUID->"40ffc1a5-ed1b-403a-839d-\
86d89849da49"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BooleanCountingFunction", "[", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.729486054101388*^9, 3.7294862143713503`*^9}},
 CellLabel->"In[1]:=",ExpressionUUID->"e0b3ed68-c0ab-4d6f-bc19-c6525e713abd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"a", "&&", "b", "&&", 
    RowBox[{"!", "c"}], "&&", 
    RowBox[{"!", "d"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"a", "&&", 
    RowBox[{"!", "b"}], "&&", "c", "&&", 
    RowBox[{"!", "d"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{"a", "&&", 
    RowBox[{"!", "b"}], "&&", 
    RowBox[{"!", "c"}], "&&", "d"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "a"}], "&&", "b", "&&", "c", "&&", 
    RowBox[{"!", "d"}]}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "a"}], "&&", "b", "&&", 
    RowBox[{"!", "c"}], "&&", "d"}], ")"}], "||", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"!", "a"}], "&&", 
    RowBox[{"!", "b"}], "&&", "c", "&&", "d"}], ")"}]}]], "Output",
 CellLabel->"Out[1]=",ExpressionUUID->"06a2415b-cffe-4ef1-bede-4ae8974bda67"]
}, Open  ]],

Cell["\<\
Such expressions can be difficult to write correctly by hand. Interestingly, \
they are also difficult to verify by means of post-conditions. Automatic code \
generation can provide a possible solution to this problem.\
\>", "Text",
 CellChangeTimes->{{3.7294862684277506`*^9, 3.7294864361239505`*^9}, {
  3.7294872128729506`*^9, 
  3.7294872431513505`*^9}},ExpressionUUID->"1faa7861-fc9e-4b86-9f24-\
94152774dd26"],

Cell["\<\
Load the FMT library and create a new model with a single package \
\[OpenCurlyDoubleQuote]Tools`\[CloseCurlyDoubleQuote] and a single operation \
with 8 input and 1 output parameters:\
\>", "Text",
 CellChangeTimes->{{3.7293173460190644`*^9, 3.7293173548058643`*^9}, {
   3.7294864521989503`*^9, 3.7294864898351507`*^9}, 3.72948655245895*^9, {
   3.7294869915551505`*^9, 
   3.7294869916175504`*^9}},ExpressionUUID->"ce44f4f6-e948-4429-8c57-\
8ab961d27ab2"],

Cell[BoxData[
 RowBox[{"<<", "FMT`"}]], "Input",
 CellChangeTimes->{{3.7191203731100197`*^9, 3.7191203746388197`*^9}},
 CellLabel->"In[2]:=",ExpressionUUID->"dc2998bc-e75e-4847-a766-1a14a4824a09"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sys", "=", 
   RowBox[{"newSystem", "[", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.71912037880402*^9, 3.7191206156588216`*^9}, {
   3.7191206498228216`*^9, 3.719120661335622*^9}, 3.7191207406928225`*^9, {
   3.7293173636888647`*^9, 3.7293173735384645`*^9}},
 CellLabel->"In[3]:=",ExpressionUUID->"65fba18f-e735-417e-8940-161af34b01e3"],

Cell[BoxData[
 RowBox[{"declarePackage", "[", 
  RowBox[{"sys", ",", "\"\<Tools`\>\""}], "]"}]], "Input",
 CellChangeTimes->{{3.71912037880402*^9, 3.7191206156588216`*^9}, {
   3.7191206498228216`*^9, 3.719120661335622*^9}, 3.7191207406928225`*^9, {
   3.7293173636888647`*^9, 3.7293173735384645`*^9}, {3.729317452369664*^9, 
   3.7293174539452643`*^9}},
 CellLabel->"In[4]:=",ExpressionUUID->"36871582-481d-478a-989d-1403b9bda884"],

Cell[BoxData[
 RowBox[{"declareOperations", "[", 
  RowBox[{"sys", ",", "\"\<Tools`\>\"", ",", "\[IndentingNewLine]", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"halfTrue", "[", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"a", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", 
      RowBox[{"b", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", 
      RowBox[{"c", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", 
      RowBox[{"d", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"e", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", 
      RowBox[{"f", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", 
      RowBox[{"g", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", 
      RowBox[{"h", "\[Rule]", 
       RowBox[{"in", "[", "boolean", "]"}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"result", "\[Rule]", 
       RowBox[{"out", "[", "boolean", "]"}]}]}], "\[IndentingNewLine]", "]"}],
     "\[IndentingNewLine]", "}"}]}], "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.71912037880402*^9, 3.7191206156588216`*^9}, {
   3.7191206498228216`*^9, 3.719120661335622*^9}, 3.7191207406928225`*^9, {
   3.7293173636888647`*^9, 3.7293173735384645`*^9}, {3.729317452369664*^9, 
   3.729317486602064*^9}, {3.7293175469508643`*^9, 3.729317595083864*^9}, {
   3.7293176275026646`*^9, 3.729317645630864*^9}, {3.7294864992459507`*^9, 
   3.7294865426279507`*^9}, {3.7294870058049507`*^9, 
   3.7294870273857503`*^9}, {3.7294872700283504`*^9, 3.7294872887311506`*^9}},
 
 CellLabel->"In[5]:=",ExpressionUUID->"df88cfc0-4719-4754-920c-357067c1662f"],

Cell["\<\
The intent is to return True when exactly 4 of the 8 input arguments are True.\
\>", "Text",
 CellChangeTimes->{{3.7294870484185505`*^9, 3.7294870810969505`*^9}, 
   3.7294873011195507`*^9},ExpressionUUID->"9cdc8fb8-5647-4d6a-a9ae-\
88ffc11a6c97"],

Cell["\<\
The function can be implemented by hand (not easy), by copy-pasting (not \
smart) or by synthesising the automatically generated expression:\
\>", "Text",
 CellChangeTimes->{{3.7294865307523503`*^9, 
  3.7294866364385505`*^9}},ExpressionUUID->"e40003c6-1e79-48cf-9d41-\
ca06307f099b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"defineOperation", "[", 
  RowBox[{
  "sys", ",", "\"\<Tools`\>\"", ",", "halfTrue", ",", "\[IndentingNewLine]", 
   RowBox[{"{", "\[IndentingNewLine]", "}"}], ",", "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"result", "=", 
    RowBox[{"Synthesize", "[", "\[IndentingNewLine]", 
     RowBox[{"BooleanCountingFunction", "[", 
      RowBox[{
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
        "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f", ",", "g", ",", 
         "h"}], "}"}]}], "]"}], "\[IndentingNewLine]", "]"}]}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.719120761628022*^9, 3.7191207935300226`*^9}, {
   3.7191234295074396`*^9, 3.71912345393704*^9}, {3.7191236921238413`*^9, 
   3.7191236925762415`*^9}, {3.7191254587414527`*^9, 
   3.7191254691778526`*^9}, {3.719125627096654*^9, 3.7191256328998537`*^9}, {
   3.7191266533046603`*^9, 3.71912665427186*^9}, {3.7191271307426634`*^9, 
   3.7191271335194635`*^9}, 3.720351102816934*^9, {3.7293179166561794`*^9, 
   3.729317928420108*^9}, {3.729317962886054*^9, 3.7293179782869453`*^9}, {
   3.729318014110153*^9, 3.7293180196035995`*^9}, {3.7294866616083508`*^9, 
   3.7294866861087503`*^9}, 3.72948693442455*^9, 3.7294870313793507`*^9, {
   3.72948708562195*^9, 3.72948708829255*^9}},
 CellLabel->"In[6]:=",ExpressionUUID->"32ffa2af-2081-46ba-8f6d-e7f68b8b0f23"],

Cell[BoxData[
 StyleBox["\<\"Grammar, names and flows OK.\"\>",
  StripOnInput->False,
  LineColor->RGBColor[0, 1, 0],
  FrontFaceColor->RGBColor[0, 1, 0],
  BackFaceColor->RGBColor[0, 1, 0],
  GraphicsColor->RGBColor[0, 1, 0],
  FontColor->RGBColor[0, 1, 0]]], "Output",
 CellLabel->"Out[6]=",ExpressionUUID->"7fb01ac4-33a8-4505-8341-1fd051f3a245"]
}, Open  ]],

Cell["\<\
The result of above expansion can be seen in the final generated code, here \
in C++:\
\>", "Text",
 CellChangeTimes->{{3.72948686770195*^9, 
  3.7294868857249503`*^9}},ExpressionUUID->"164aaf60-7921-47c6-b443-\
1625df325506"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"generateCode", "[", 
   RowBox[{"sys", ",", "\"\<C++\>\""}], "]"}], "[", "\"\<tools.cpp\>\"", 
  "]"}]], "Input",
 CellChangeTimes->{{3.72948689004615*^9, 3.7294868957421503`*^9}, {
  3.7294869486265507`*^9, 3.72948695201175*^9}},
 CellLabel->"In[7]:=",ExpressionUUID->"06b40c5e-8328-49a2-80c0-2a75346ffa9b"],

Cell[BoxData["\<\"#include \\\"tools.h\\\"\\n\\nusing namespace \
tools;\\n\\n// Operation definitions.\\n\\nvoid tools::half_true(bool a, bool \
b, bool c, bool d, bool e, bool f, bool g, bool\\n    h, bool & /* out */ \
result)\\n{\\n    result = (a && b && c && d && !e && !f && !g && !h) || (a \
&& b && c && !d &&\\n        e && !f && !g && !h) || (a && b && c && !d && !e \
&& f && !g && !h) || (a\\n        && b && c && !d && !e && !f && g && !h) || \
(a && b && c && !d && !e && !f\\n        && !g && h) || (a && b && !c && d && \
e && !f && !g && !h) || (a && b &&\\n        !c && d && !e && f && !g && !h) \
|| (a && b && !c && d && !e && !f && g &&\\n        !h) || (a && b && !c && d \
&& !e && !f && !g && h) || (a && b && !c && !d\\n        && e && f && !g && \
!h) || (a && b && !c && !d && e && !f && g && !h) ||\\n        (a && b && !c \
&& !d && e && !f && !g && h) || (a && b && !c && !d && !e\\n        && f && g \
&& !h) || (a && b && !c && !d && !e && f && !g && h) || (a &&\\n        b && \
!c && !d && !e && !f && g && h) || (a && !b && c && d && e && !f &&\\n        \
!g && !h) || (a && !b && c && d && !e && f && !g && !h) || (a && !b && c\\n   \
     && d && !e && !f && g && !h) || (a && !b && c && d && !e && !f && !g \
&&\\n        h) || (a && !b && c && !d && e && f && !g && !h) || (a && !b && \
c && !d\\n        && e && !f && g && !h) || (a && !b && c && !d && e && !f && \
!g && h) ||\\n        (a && !b && c && !d && !e && f && g && !h) || (a && !b \
&& c && !d && !e\\n        && f && !g && h) || (a && !b && c && !d && !e && \
!f && g && h) || (a &&\\n        !b && !c && d && e && f && !g && !h) || (a \
&& !b && !c && d && e && !f &&\\n        g && !h) || (a && !b && !c && d && e \
&& !f && !g && h) || (a && !b && !c\\n        && d && !e && f && g && !h) || \
(a && !b && !c && d && !e && f && !g && h)\\n        || (a && !b && !c && d \
&& !e && !f && g && h) || (a && !b && !c && !d &&\\n        e && f && g && \
!h) || (a && !b && !c && !d && e && f && !g && h) || (a &&\\n        !b && !c \
&& !d && e && !f && g && h) || (a && !b && !c && !d && !e && f\\n        && g \
&& h) || (!a && b && c && d && e && !f && !g && !h) || (!a && b &&\\n        \
c && d && !e && f && !g && !h) || (!a && b && c && d && !e && !f && g &&\\n   \
     !h) || (!a && b && c && d && !e && !f && !g && h) || (!a && b && c && !d\
\\n        && e && f && !g && !h) || (!a && b && c && !d && e && !f && g && \
!h) ||\\n        (!a && b && c && !d && e && !f && !g && h) || (!a && b && c \
&& !d && !e\\n        && f && g && !h) || (!a && b && c && !d && !e && f && \
!g && h) || (!a &&\\n        b && c && !d && !e && !f && g && h) || (!a && b \
&& !c && d && e && f &&\\n        !g && !h) || (!a && b && !c && d && e && !f \
&& g && !h) || (!a && b && !c\\n        && d && e && !f && !g && h) || (!a && \
b && !c && d && !e && f && g && !h)\\n        || (!a && b && !c && d && !e && \
f && !g && h) || (!a && b && !c && d &&\\n        !e && !f && g && h) || (!a \
&& b && !c && !d && e && f && g && !h) || (!a\\n        && b && !c && !d && e \
&& f && !g && h) || (!a && b && !c && !d && e && !f\\n        && g && h) || \
(!a && b && !c && !d && !e && f && g && h) || (!a && !b &&\\n        c && d \
&& e && f && !g && !h) || (!a && !b && c && d && e && !f && g &&\\n        \
!h) || (!a && !b && c && d && e && !f && !g && h) || (!a && !b && c && d\\n   \
     && !e && f && g && !h) || (!a && !b && c && d && !e && f && !g && h) \
||\\n        (!a && !b && c && d && !e && !f && g && h) || (!a && !b && c && \
!d && e\\n        && f && g && !h) || (!a && !b && c && !d && e && f && !g && \
h) || (!a &&\\n        !b && c && !d && e && !f && g && h) || (!a && !b && c \
&& !d && !e && f &&\\n        g && h) || (!a && !b && !c && d && e && f && g \
&& !h) || (!a && !b && !c\\n        && d && e && f && !g && h) || (!a && !b \
&& !c && d && e && !f && g && h)\\n        || (!a && !b && !c && d && !e && f \
&& g && h) || (!a && !b && !c && !d &&\\n        e && f && g && \
h);\\n}\\n\"\>"], "Output",
 CellLabel->"Out[7]=",ExpressionUUID->"b25dae27-1f7b-409e-b3ee-80f78fa5c8fe"]
}, Open  ]],

Cell["\<\
The Synthesize grammar structure is a meta-statement that is replaced by its \
evaluated argument where it is used. It can be used anywhere in the operation\
\[CloseCurlyQuote]s body and can expand to arbitrary complex code, not just \
single expressions.\
\>", "Text",
 CellChangeTimes->{{3.7294866942529507`*^9, 3.7294868587425504`*^9}, {
  3.7294943696299534`*^9, 
  3.7294943791995697`*^9}},ExpressionUUID->"2ff15674-1aa0-4690-8e85-\
82fcc4bbd38f"],

Cell["\<\
Code synthesis is a powerful tool, but it comes with its own set traps - \
namely, the evaluation of Synthesize can interfere with symbols used in the \
model, it can also rely on constructs that already have some meaning in \
Wolfram. Special care should be taken with regard to assignments, which are \
normally protected inside operation bodies, but not during synthesis. For \
this reason, the assignmentT and other FMT-defined tags should be used \
instead of regular operations to avoid any unwanted interference.\
\>", "Text",
 CellChangeTimes->{{3.7294886968189144`*^9, 3.729488698758012*^9}, {
  3.7294887309854937`*^9, 3.7294888784507685`*^9}, {3.729488930299859*^9, 
  3.7294890091411357`*^9}, {3.729490076181776*^9, 3.7294901222453823`*^9}, {
  3.7294943915605316`*^9, 
  3.729494460264011*^9}},ExpressionUUID->"23eb58b6-200c-49dc-b104-\
db59f5c6dff5"],

Cell["\<\
The following example avoids the assignment, array access and compound \
expression problems when generating prime number array initialization:\
\>", "Text",
 CellChangeTimes->{{3.7294890130835*^9, 3.7294890562723975`*^9}, {
  3.7294901277959366`*^9, 3.7294901403711944`*^9}, {3.7294932115521984`*^9, 
  3.7294932135329027`*^9}},ExpressionUUID->"1828ad2c-1e41-4755-8522-\
2b8d9093904d"],

Cell[BoxData[
 RowBox[{"declarePackage", "[", 
  RowBox[{"sys", ",", "\"\<Primes`\>\""}], "]"}]], "Input",
 CellChangeTimes->{{3.729489059288206*^9, 3.7294890837506804`*^9}, {
  3.7294892509498253`*^9, 3.7294892544068623`*^9}},
 CellLabel->"In[8]:=",ExpressionUUID->"d2522380-a05a-45ff-9f40-03c9d063c756"],

Cell[BoxData[
 RowBox[{"defineDataTypes", "[", 
  RowBox[{"sys", ",", "\"\<Primes`\>\"", ",", "\[IndentingNewLine]", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"arrayOfInts", "\[Rule]", 
     RowBox[{"array", "[", 
      RowBox[{"integer", ",", "20"}], "]"}]}], "\[IndentingNewLine]", "}"}]}],
   "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.7294890995794277`*^9, 3.7294891253701625`*^9}, {
  3.729489314827983*^9, 3.7294893153861504`*^9}},
 CellLabel->"In[9]:=",ExpressionUUID->"528bc23e-32df-40d5-9b03-1ca0788fb0c1"],

Cell[BoxData[
 RowBox[{"declareOperations", "[", 
  RowBox[{"sys", ",", "\"\<Primes`\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"someOperation", "[", "]"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7294891275408134`*^9, 3.729489165067068*^9}},
 CellLabel->"In[10]:=",ExpressionUUID->"7fae673f-2f5d-4215-b1c9-140b4ef40664"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"defineOperation", "[", 
  RowBox[{
  "sys", ",", "\"\<Primes`\>\"", ",", "someOperation", ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{"a", "\[Rule]", "arrayOfInts"}], "\[IndentingNewLine]", "}"}], 
   ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"Synthesize", "[", "\[IndentingNewLine]", 
    RowBox[{"FMTSymbols`compoundT", "@@", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"FMTSymbols`assignmentT", "[", 
        RowBox[{
         RowBox[{"FMTSymbols`partT", "[", 
          RowBox[{"a", ",", "i"}], "]"}], ",", 
         RowBox[{"Prime", "[", 
          RowBox[{"i", "+", "1"}], "]"}]}], "]"}], ",", " ", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "=", 
         RowBox[{"Prime", "[", 
          RowBox[{"i", "+", "1"}], "]"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i", ",", "0", ",", "9"}], "}"}]}], "\[IndentingNewLine]", 
      "]"}]}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 CellChangeTimes->{{3.7294891560183544`*^9, 3.729489219136284*^9}, {
  3.729489325502184*^9, 3.729489335001033*^9}, {3.729489412040368*^9, 
  3.7294894152986946`*^9}, {3.729489696554817*^9, 3.7294897512922907`*^9}, {
  3.7294897880399647`*^9, 3.729489905292689*^9}, {3.72949016213137*^9, 
  3.7294902526994257`*^9}},
 CellLabel->"In[11]:=",ExpressionUUID->"5b5d73c1-ed33-4e25-8055-65923a0404bd"],

Cell[BoxData[
 StyleBox["\<\"Grammar, names and flows OK.\"\>",
  StripOnInput->False,
  LineColor->RGBColor[0, 1, 0],
  FrontFaceColor->RGBColor[0, 1, 0],
  BackFaceColor->RGBColor[0, 1, 0],
  GraphicsColor->RGBColor[0, 1, 0],
  FontColor->RGBColor[0, 1, 0]]], "Output",
 CellLabel->"Out[11]=",ExpressionUUID->"7d034364-a816-4069-97c4-e96d89d2ecb6"]
}, Open  ]],

Cell["\<\
In the above implementation the Table and Prime functions are standard \
Mathematica functions that are responsible for the actual synthesis logic. \
The other constructs (from the FMTSymbols` package) are needed instead of \
their standard versions, because when Synthesize evaluates its body, the \
a[[i]] would not be recognized as a valid construct, assignment would \
interfere with visible symbols and regular compound expression would abandon \
everything except the last component.\
\>", "Text",
 CellChangeTimes->{{3.7294932835777893`*^9, 3.729493560230074*^9}, {
  3.7294944913590136`*^9, 3.729494517475911*^9}, {3.729494564777602*^9, 
  3.72949459190948*^9}},ExpressionUUID->"ae0eb124-88f7-4265-a7a2-\
311912b1ca66"],

Cell["The final generated source code for this package is:", "Text",
 CellChangeTimes->{{3.729493231114707*^9, 3.729493251503209*^9}, {
  3.7294946329373035`*^9, 
  3.7294946361352415`*^9}},ExpressionUUID->"d39e3260-ebd6-4df1-a815-\
4ef068e604f7"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"generateCode", "[", 
   RowBox[{"sys", ",", "\"\<C++\>\""}], "]"}], "[", "\"\<primes.cpp\>\"", 
  "]"}]], "Input",
 CellChangeTimes->{{3.72948933892621*^9, 3.729489390775242*^9}, {
   3.7294900131714754`*^9, 3.7294900260707655`*^9}, {3.729490865807809*^9, 
   3.729490869991809*^9}, {3.729492106423809*^9, 3.729492116435809*^9}, {
   3.7294925360329227`*^9, 3.729492550632741*^9}, {3.729494126581328*^9, 
   3.729494136721328*^9}, 3.7294946408865504`*^9},
 CellLabel->"In[12]:=",ExpressionUUID->"90b0caef-ff60-47a1-b6c5-8a95833192ce"],

Cell[BoxData["\<\"#include \\\"primes.h\\\"\\n\\nusing namespace \
primes;\\n\\n// Operation definitions.\\n\\nvoid primes::some_operation()\\n{\
\\n    array_of_ints a;\\n    \\n    a[0] = 2;\\n    a[1] = 3;\\n    a[2] = \
5;\\n    a[3] = 7;\\n    a[4] = 11;\\n    a[5] = 13;\\n    a[6] = 17;\\n    \
a[7] = 19;\\n    a[8] = 23;\\n    a[9] = 29;\\n}\\n\"\>"], "Output",
 CellLabel->"Out[12]=",ExpressionUUID->"fd4b570a-71b3-47e7-9f90-dc734f48d972"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{808, 911},
WindowMargins->{{36, Automatic}, {40, Automatic}},
FrontEndVersion->"11.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 5, \
2018)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 273, 4, 54, "Subsection",ExpressionUUID->"61718ed3-bdbb-4523-bbc2-370a0fc7fbc4"],
Cell[856, 28, 515, 9, 58, "Text",ExpressionUUID->"1db54fa5-be64-46d3-b176-848795c5d7f3"],
Cell[1374, 39, 673, 11, 104, "Text",ExpressionUUID->"40ffc1a5-ed1b-403a-839d-86d89849da49"],
Cell[CellGroupData[{
Cell[2072, 54, 338, 7, 30, "Input",ExpressionUUID->"e0b3ed68-c0ab-4d6f-bc19-c6525e713abd"],
Cell[2413, 63, 833, 26, 56, "Output",ExpressionUUID->"06a2415b-cffe-4ef1-bede-4ae8974bda67"]
}, Open  ]],
Cell[3261, 92, 425, 8, 58, "Text",ExpressionUUID->"1faa7861-fc9e-4b86-9f24-94152774dd26"],
Cell[3689, 102, 468, 9, 58, "Text",ExpressionUUID->"ce44f4f6-e948-4429-8c57-8ab961d27ab2"],
Cell[4160, 113, 196, 3, 30, "Input",ExpressionUUID->"dc2998bc-e75e-4847-a766-1a14a4824a09"],
Cell[4359, 118, 376, 7, 30, "Input",ExpressionUUID->"65fba18f-e735-417e-8940-161af34b01e3"],
Cell[4738, 127, 432, 7, 30, "Input",ExpressionUUID->"36871582-481d-478a-989d-1403b9bda884"],
Cell[5173, 136, 1712, 33, 199, "Input",ExpressionUUID->"df88cfc0-4719-4754-920c-357067c1662f"],
Cell[6888, 171, 257, 5, 35, "Text",ExpressionUUID->"9cdc8fb8-5647-4d6a-a9ae-88ffc11a6c97"],
Cell[7148, 178, 294, 6, 58, "Text",ExpressionUUID->"e40003c6-1e79-48cf-9d41-ca06307f099b"],
Cell[CellGroupData[{
Cell[7467, 188, 1424, 26, 178, "Input",ExpressionUUID->"32ffa2af-2081-46ba-8f6d-e7f68b8b0f23"],
Cell[8894, 216, 349, 8, 34, "Output",ExpressionUUID->"7fb01ac4-33a8-4505-8341-1fd051f3a245"]
}, Open  ]],
Cell[9258, 227, 236, 6, 35, "Text",ExpressionUUID->"164aaf60-7921-47c6-b443-1625df325506"],
Cell[CellGroupData[{
Cell[9519, 237, 343, 7, 30, "Input",ExpressionUUID->"06b40c5e-8328-49a2-80c0-2a75346ffa9b"],
Cell[9865, 246, 4104, 52, 1190, "Output",ExpressionUUID->"b25dae27-1f7b-409e-b3ee-80f78fa5c8fe"]
}, Open  ]],
Cell[13984, 301, 462, 9, 81, "Text",ExpressionUUID->"2ff15674-1aa0-4690-8e85-82fcc4bbd38f"],
Cell[14449, 312, 874, 14, 127, "Text",ExpressionUUID->"23eb58b6-200c-49dc-b104-db59f5c6dff5"],
Cell[15326, 328, 396, 7, 58, "Text",ExpressionUUID->"1828ad2c-1e41-4755-8522-2b8d9093904d"],
Cell[15725, 337, 305, 5, 30, "Input",ExpressionUUID->"d2522380-a05a-45ff-9f40-03c9d063c756"],
Cell[16033, 344, 545, 10, 115, "Input",ExpressionUUID->"528bc23e-32df-40d5-9b03-1ca0788fb0c1"],
Cell[16581, 356, 327, 6, 30, "Input",ExpressionUUID->"7fae673f-2f5d-4215-b1c9-140b4ef40664"],
Cell[CellGroupData[{
Cell[16933, 366, 1553, 34, 283, "Input",ExpressionUUID->"5b5d73c1-ed33-4e25-8055-65923a0404bd"],
Cell[18489, 402, 350, 8, 34, "Output",ExpressionUUID->"7d034364-a816-4069-97c4-e96d89d2ecb6"]
}, Open  ]],
Cell[18854, 413, 737, 12, 127, "Text",ExpressionUUID->"ae0eb124-88f7-4265-a7a2-311912b1ca66"],
Cell[19594, 427, 247, 4, 35, "Text",ExpressionUUID->"d39e3260-ebd6-4df1-a815-4ef068e604f7"],
Cell[CellGroupData[{
Cell[19866, 435, 569, 10, 30, "Input",ExpressionUUID->"90b0caef-ff60-47a1-b6c5-8a95833192ce"],
Cell[20438, 447, 449, 5, 476, "Output",ExpressionUUID->"fd4b570a-71b3-47e7-9f90-dc734f48d972"]
}, Open  ]]
}, Open  ]]
}
]
*)

